﻿-- locals, helperfunctions
local string_gsub, string_format = string.gsub, string.format
local table_sort, table_concat = table.sort, table.concat
local function Printmsg(...) print("|cFF33FF99Ailo|r:", ...) end

-- LibDataBroker
local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
if not ldb then return end
local LibQTip = LibStub('LibQTip-1.0')

local Ailo = {}

AiloDB = AiloDB or {}

AiloDB.chars = AiloDB.chars or {}
AiloDB.raids = AiloDB.raids or {}
AiloDB.minimapIcon = AiloDB.minimapIcon or {}

local AiloLDB = ldb:NewDataObject("Ailo", {
	type = "data source",
	text = "Ailo",
	icon = "Interface\\Icons\\Achievement_Dungeon_UlduarRaid_Archway_01.png",
	OnClick = function(clickedframe, button)
		if button == "RightButton" then Ailo:OutputToChat() else Ailo:ManualPlayerUpdate() end
	end,
    OnEnter = function(tt)
        local tooltip = LibQTip:Acquire("AiloTolltip", 1, "LEFT") 
        Ailo.tooltip = tooltip
        Ailo:PrepareTooltip(tooltip) 
        tooltip:SmartAnchorTo(tt)
        tooltip:Show()
    end,
    OnLeave = function(tt)
        LibQTip:Release(Ailo.tooltip)
        Ailo.tooltip = nil
    end,
})

local LDBIcon = LibStub("LibDBIcon-1.0",true)
if LDBIcon then 
  LDBIcon:Register("Ailo", AiloLDB, AiloDB.minimapIcon)
end


local frame = CreateFrame("Frame")
local function onEventFunc(self, event, ...)
	if event == "UPDATE_INSTANCE_INFO" then
		self:UpdateRaidIDDB()
	end
end
frame:RegisterEvent("UPDATE_INSTANCE_INFO")
frame:SetScript("OnEvent", onEventFunc)
frame.InstanceNameFormatString = "%s%d%s"
local raidorder= {}
function Ailo:PrepareTooltip( tooltip )
    --[[[ Cell are just colored green/red
               ToC       VoA
            10     25   10  25
          hc nhc hc nhc nhc nhc
    Char1 [] [ ] [] [ ] [ ] [ ]
    Char2 [] [ ] [] [ ] [ ] [ ]
    Char3 [] [ ] [] [ ] [ ] [ ]
    ]]--
    tooltip:AddHeader("Raid") -- Raid
    tooltip:AddHeader("Size") -- Size
    tooltip:AddHeader("Diff") -- Heroic

    local raidsd = AiloDB.raids
    local raid, size, difficulties,  difficulty, colcount
    local colcount, numdifficulties
    for raid, sizes in pairs( raidsdb ) do
        colcount = 0 -- Span needed for the 'Raid' cell above the 'Size' cells

        for size, difficulties in pairs( sizes ) do
            numdifficulties = 0 -- Span needed for the 'Size' cell above the 'Difficulty' cells

            for difficulty, _ in pairs( difficulties ) do
                colcount = colcount +1
                numdifficulties = numdifficulties +1

                lastcolumn = tooltip:AddColumn( "CENTER" )
                tooltip:SetCell(3, lastcolumn, difficulty)

                raidorder[(string_format(frame.InstanceNameFormatString, raid, size, difficulty))] = lastcolumn

            end
            tooltip:SetCell(2, lastcolumn - numdifficulties+1, size, numdifficulties)
        end
        tooltip:SetCell(1, lastcolumn - colcount+1, raid, colcount)
    end
    local currentPlayer, raids, currentRaid, lastline
    local charsdb = AiloDB.chars
    for currentPlayer, raids in pairs( charsdb ) do 
        lastline = tooltip:AddLine( currentPlayer )
        for i = tooltip:GetColumnCount(),2,-1 do
            tooltip:SetCell( lastline, i, "") -- hackety hack :(
            tooltip:SetCellColor( lastline, i, 0, 1, 0 )
        end
        
        for currentRaid, _ in pairs( raids ) do
            tooltip:SetCellColor( lastline, raidorder[currentRaid], 1, 0, 0 )
        end
    end
end


function Ailo:ExtendRaidTable( name, size, heroic)
    local raid = { size = { heroic = true } }
    AiloDB.raids[name] = AiloDB.raids[name] or {}
    AiloDB.raids[name][size] =  AiloDB.raids[name][size] or {}
    AiloDB.raids[name][size][heroic] = true
end


function Ailo:ManualPlayerUpdate()
    local thisplayer, currentRaid = UnitName("player")
    if not AiloDB.chars[thisplayer] then 
        AiloDB.chars[thisplayer] = {}
    else 
        for currentRaid, _ in pairs(AiloDB.chars[thisplayer]) do
            AiloDB.chars[thisplayer][currentRaid] = nil
        end
    end
    frame:UpdateRaidIDDB()
end

function Ailo:OutputToChat()
    local currentPlayer, curPlayerRaids, currentRaid, currentRaidExpire, numRaids, i
    local savedRaids, now = {}, time()
    local chars = AiloDB.chars
    for currentPlayer, curPlayerRaids in pairs(chars) do 
        numRaids = 0
        for currentRaid, currentRaidExpire in pairs(curPlayerRaids) do
            if now > currentRaidExpire then
                AiloDB.chars[currentPlayer][currentRaid] = nil
            else
                numRaids = numRaids+1 
                savedRaids[numRaids] = currentRaid
            end
        end
        if numRaids > 0 then 
            table_sort(savedRaids)
            Printmsg(currentPlayer..": "..table_concat(savedRaids, ", ")) 
            for numRaids,_ in ipairs(savedRaids) do 
                savedRaids[numRaids] = nil 
            end
        end
    end
end

function frame:UpdateRaidIDDB()
    local now, thisplayer, index = time(), UnitName("player")
    if not AiloDB.chars[thisplayer] then AiloDB.chars[thisplayer] = {} end
	for index=1,GetNumSavedInstances() do
	    --instanceName, instanceID, instanceReset, instanceDifficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName = GetSavedInstanceInfo(index)
        instanceName, _, instanceReset, instanceDifficulty, locked, _, _, isRaid, maxPlayers, _ = GetSavedInstanceInfo(index)
		if locked and isRaid then
		    local instanceNameAbbr = string_gsub(instanceName, "(%a)[%l%p]*[%s%-]*", "%1")
		    local heroicOrNot      = instanceDifficulty > 2 and "hc" or "nhc"
		    local instanceSring    = string_format(self.InstanceNameFormatString, instanceNameAbbr, maxPlayers, heroicOrNot)
		    Ailo:ExtendRaidTable(instanceNameAbbr, maxPlayers, heroicOrNot)
		    AiloDB.chars[thisplayer][instanceSring] = now+instanceReset
		end
	end
end

